@@ -2,7 +2,6 @@
 Routes and sub-resources for the /logs resource
 """
 from flask import Blueprint, jsonify, request
-import os
 import subprocess
 import json
 import csle_common.constants.constants as constants
@@ -10,9 +9,9 @@ import csle_rest_api.constants.constants as api_constants
 import csle_collector.constants.constants as collector_constants
 import csle_ryu.constants.constants as ryu_constants
 import csle_rest_api.util.rest_api_util as rest_api_util
-from csle_common.dao.emulation_config.config import Config
 from csle_common.metastore.metastore_facade import MetastoreFacade
 from csle_common.util.emulation_util import EmulationUtil
+from csle_cluster.cluster_manager.cluster_controller import ClusterController
 
 # Creates a blueprint "sub application" of the main REST app
 logs_bp = Blueprint(
@@ -20,7 +19,7 @@ logs_bp = Blueprint(
     url_prefix=f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.LOGS_RESOURCE}")
 
 
-@logs_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
+@logs_bp.route("", methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
 def logs():
     """
     The /logs resource.
@@ -32,18 +31,11 @@ def logs():
     authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
     if authorized is not None:
         return authorized
-
-    config = Config.get_current_config()
-    path = config.default_log_dir
-    log_files = []
-    for f in os.listdir(path):
-        item = os.path.join(path, f)
-        if os.path.isfile(item):
-            log_files.append(item)
-    if len(log_files) > 20:
-        log_files = log_files[0:20]
-    data = log_files
-    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
+    json_data = json.loads(request.data)
+    if api_constants.MGMT_WEBAPP.IP_PROPERTY not in json_data:
+        return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
+    ip = json_data[api_constants.MGMT_WEBAPP.IP_PROPERTY]
+    data_dict = ClusterController.get_csle_log_files(ip=ip, port=constants.GRPC_SERVERS.CLUSTER_MANAGER_PORT)
     response = jsonify(data_dict)
     response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
     return response, constants.HTTPS.OK_STATUS_CODE
@@ -62,51 +54,130 @@ def docker_stats_manager_logs():
     authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
     if authorized is not None:
         return authorized
-
-    config = Config.get_current_config()
-    path = config.docker_stats_manager_log_dir + config.docker_stats_manager_log_file
-
-    if os.path.exists(path):
-        with open(path, 'r') as fp:
-            data = fp.readlines()
-            tail = data[-100:]
-            data = tail
-    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
+    json_data = json.loads(request.data)
+    if api_constants.MGMT_WEBAPP.IP_PROPERTY not in json_data:
+        return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
+    ip = json_data[api_constants.MGMT_WEBAPP.IP_PROPERTY]
+    data_dict = ClusterController.get_docker_statsmanager_logs(ip=ip, port=constants.GRPC_SERVERS.CLUSTER_MANAGER_PORT)
     response = jsonify(data_dict)
     response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
     return response, constants.HTTPS.OK_STATUS_CODE
 
 
 @logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.PROMETHEUS_RESOURCE}",
-               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
+               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
 def prometheus_logs():
     """
     The /logs/prometheus resource.
 
-    :return: The logs of the docker stats manager
+    :return: The Prometheus logs
     """
 
     # Check that token is valid
     authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
     if authorized is not None:
         return authorized
+    json_data = json.loads(request.data)
+    if api_constants.MGMT_WEBAPP.IP_PROPERTY not in json_data:
+        return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
+    ip = json_data[api_constants.MGMT_WEBAPP.IP_PROPERTY]
+    data_dict = ClusterController.get_prometheus_logs(ip=ip, port=constants.GRPC_SERVERS.CLUSTER_MANAGER_PORT)
+    response = jsonify(data_dict)
+    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
+    return response, constants.HTTPS.OK_STATUS_CODE
 
-    config = Config.get_current_config()
-    path = config.prometheus_log_file
 
-    if os.path.exists(path):
-        with open(path, 'r') as fp:
-            data = fp.readlines()
-            tail = data[-100:]
-            data = tail
-    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
+@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.NGINX_RESOURCE}",
+               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
+def nginx_logs():
+    """
+    The /logs/nginx resource.
+
+    :return: The nginx logs
+    """
+
+    # Check that token is valid
+    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
+    if authorized is not None:
+        return authorized
+    json_data = json.loads(request.data)
+    if api_constants.MGMT_WEBAPP.IP_PROPERTY not in json_data:
+        return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
+    ip = json_data[api_constants.MGMT_WEBAPP.IP_PROPERTY]
+    data_dict = ClusterController.get_nginx_logs(ip=ip, port=constants.GRPC_SERVERS.CLUSTER_MANAGER_PORT)
+    response = jsonify(data_dict)
+    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
+    return response, constants.HTTPS.OK_STATUS_CODE
+
+
+@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.POSTGRESQL_RESOURCE}",
+               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
+def postgresql_logs():
+    """
+    The /logs/postgresql resource.
+
+    :return: The PostgreSQL logs
+    """
+    # Check that token is valid
+    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
+    if authorized is not None:
+        return authorized
+    json_data = json.loads(request.data)
+    if api_constants.MGMT_WEBAPP.IP_PROPERTY not in json_data:
+        return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
+    ip = json_data[api_constants.MGMT_WEBAPP.IP_PROPERTY]
+    data_dict = ClusterController.get_postgresql_logs(ip=ip, port=constants.GRPC_SERVERS.CLUSTER_MANAGER_PORT)
+    response = jsonify(data_dict)
+    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
+    return response, constants.HTTPS.OK_STATUS_CODE
+
+
+@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.FLASK_RESOURCE}",
+               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
+def flask_logs():
+    """
+    The /logs/flask resource.
+
+    :return: The Flask logs
+    """
+    # Check that token is valid
+    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
+    if authorized is not None:
+        return authorized
+    json_data = json.loads(request.data)
+    if api_constants.MGMT_WEBAPP.IP_PROPERTY not in json_data:
+        return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
+    ip = json_data[api_constants.MGMT_WEBAPP.IP_PROPERTY]
+    data_dict = ClusterController.get_flask_logs(ip=ip, port=constants.GRPC_SERVERS.CLUSTER_MANAGER_PORT)
+    response = jsonify(data_dict)
+    response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
+    return response, constants.HTTPS.OK_STATUS_CODE
+
+
+@logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.DOCKER_RESOURCE}",
+               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
+def docker_logs():
+    """
+    The /logs/docker resource.
+
+    :return: The Docker logs
+    """
+    # Check that token is valid
+    authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
+    if authorized is not None:
+        return authorized
+    json_data = json.loads(request.data)
+    if api_constants.MGMT_WEBAPP.IP_PROPERTY not in json_data:
+        return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
+    ip = json_data[api_constants.MGMT_WEBAPP.IP_PROPERTY]
+    data_dict = ClusterController.get_docker_logs(ip=ip, port=constants.GRPC_SERVERS.CLUSTER_MANAGER_PORT)
     response = jsonify(data_dict)
     response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
     return response, constants.HTTPS.OK_STATUS_CODE
 
 
 @logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.NODE_EXPORTER_RESOURCE}",
-               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
+               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
 def node_exporter_logs():
     """
     The /logs/node-exporter resource.
@@ -118,23 +189,18 @@ def node_exporter_logs():
     authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
     if authorized is not None:
         return authorized
-
-    config = Config.get_current_config()
-    path = config.node_exporter_log_file
-
-    if os.path.exists(path):
-        with open(path, 'r') as fp:
-            data = fp.readlines()
-            tail = data[-100:]
-            data = tail
-    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
+    json_data = json.loads(request.data)
+    if api_constants.MGMT_WEBAPP.IP_PROPERTY not in json_data:
+        return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
+    ip = json_data[api_constants.MGMT_WEBAPP.IP_PROPERTY]
+    data_dict = ClusterController.get_node_exporter_logs(ip=ip, port=constants.GRPC_SERVERS.CLUSTER_MANAGER_PORT)
     response = jsonify(data_dict)
     response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
     return response, constants.HTTPS.OK_STATUS_CODE
 
 
 @logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.CADVISOR_RESOURCE}",
-               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
+               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
 def cadvisor_logs():
     """
     The /logs/cadvisor resource.
@@ -146,21 +212,18 @@ def cadvisor_logs():
     authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
     if authorized is not None:
         return authorized
-
-    cmd = constants.COMMANDS.CADVISOR_LOGS
-    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
-    (output, err) = p.communicate()
-    output = output.decode("utf-8")
-    output = output.split("\n")[-100:]
-    data = output
-    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
+    json_data = json.loads(request.data)
+    if api_constants.MGMT_WEBAPP.IP_PROPERTY not in json_data:
+        return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
+    ip = json_data[api_constants.MGMT_WEBAPP.IP_PROPERTY]
+    data_dict = ClusterController.get_cadvisor_logs(ip=ip, port=constants.GRPC_SERVERS.CLUSTER_MANAGER_PORT)
     response = jsonify(data_dict)
     response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
     return response, constants.HTTPS.OK_STATUS_CODE
 
 
 @logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.PGADMIN_RESOURCE}",
-               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
+               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
 def pgadmin_logs():
     """
     The /logs/pgadmin resource.
@@ -172,40 +235,33 @@ def pgadmin_logs():
     authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
     if authorized is not None:
         return authorized
-
-    cmd = constants.COMMANDS.PGADMIN_LOGS
-    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
-    (output, err) = p.communicate()
-    output = output.decode("utf-8")
-    output = output.split("\n")[-100:]
-    data = output
-    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
+    json_data = json.loads(request.data)
+    if api_constants.MGMT_WEBAPP.IP_PROPERTY not in json_data:
+        return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
+    ip = json_data[api_constants.MGMT_WEBAPP.IP_PROPERTY]
+    data_dict = ClusterController.get_pgadmin_logs(ip=ip, port=constants.GRPC_SERVERS.CLUSTER_MANAGER_PORT)
     response = jsonify(data_dict)
     response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
     return response, constants.HTTPS.OK_STATUS_CODE
 
 
 @logs_bp.route(f"{constants.COMMANDS.SLASH_DELIM}{api_constants.MGMT_WEBAPP.GRAFANA_RESOURCE}",
-               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_GET])
+               methods=[api_constants.MGMT_WEBAPP.HTTP_REST_POST])
 def grafana_logs():
     """
     The /lofgs/grafana resource.
 
     :return: The logs of the docker stats manager
     """
-
     # Check that token is valid
     authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
     if authorized is not None:
         return authorized
-
-    cmd = constants.COMMANDS.GRAFANA_LOGS
-    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
-    (output, err) = p.communicate()
-    output = output.decode("utf-8")
-    output = output.split("\n")[-100:]
-    data = output
-    data_dict = {api_constants.MGMT_WEBAPP.LOGS_PROPERTY: data}
+    json_data = json.loads(request.data)
+    if api_constants.MGMT_WEBAPP.IP_PROPERTY not in json_data:
+        return jsonify({}), constants.HTTPS.BAD_REQUEST_STATUS_CODE
+    ip = json_data[api_constants.MGMT_WEBAPP.IP_PROPERTY]
+    data_dict = ClusterController.get_grafana_logs(ip=ip, port=constants.GRPC_SERVERS.CLUSTER_MANAGER_PORT)
     response = jsonify(data_dict)
     response.headers.add(api_constants.MGMT_WEBAPP.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
     return response, constants.HTTPS.OK_STATUS_CODE
@@ -219,7 +275,6 @@ def container_logs():
 
     :return: The logs of a specific container
     """
-
     # Check that token is valid
     authorized = rest_api_util.check_if_user_is_authorized(request=request, requires_admin=True)
     if authorized is not None:
